from .MachineLearning import UETMachineLearning
import pandas
from sklearn.cluster import AgglomerativeClustering
import operator


class UETClustering(UETMachineLearning):

    def __init__(self, n_cluster=5, modelname='', X=pandas.DataFrame(), filepath='',columnX=[]):
        super().__init__()
        self.n_cluster = n_cluster
        self.modelname = modelname
        self.dataset = X
        self.X = X[X.columns[0:5]]
        self.type = 'clustering'
        self.filepath = filepath

    def fit(self):
        self.model = AgglomerativeClustering(n_clusters=self.n_cluster)
        self.transformData()
        self.model.fit(self.X)
        return self.model

    def predict(self, X):
        X = self.scaler.transform(X)
        if self.model is None:
            self.loadModel()
        result = self.model.predict(X)
        return result

    def getLabels(self):
        if self.model is None:
            self.fit()
        labels = self.model.labels_
        return labels

    def bestResult(self):
        if self.model is None:
            self.fit()
        data = pandas.DataFrame(data=self.X, columns=['view', 'post', 'forumview', 'forumpost', 'successsubmission'])
        data['group'] = self.getLabels()
        return data

    def labelGrade(self, labels):
        data = pandas.DataFrame(data=self.dataset)
        column = {
            'w7_view': 'view',
            'w7_post': 'post',
            'w7_forumview': 'forumview',
            'w7_forumpost': 'forumpost',
            'w7_successsubmission': 'successsubmission',
            'w7_grade': 'grade',
            'w15_view': 'view',
            'w15_post': 'post',
            'w15_forumview': 'forumview',
            'w15_forumpost': 'forumpost',
            'w15_successsubmission': 'successsubmission',
            'w15_grade': 'grade'
        }
        data = data.rename(index=str, columns=column)
        data['group'] = self.getLabels()
        l = self.label(self.n_cluster, labels, data)
        data['group'] = l

        return data

    def label(self, ncluster, labels, data):
        tb = {}
        i = 0
        while (i < ncluster):
            temp = data[data.group == i]
            tb[i] = temp.grade.mean()
            i += 1
        sorted_x = sorted(tb.items(), key=operator.itemgetter(1))
        i = 0
        while i < ncluster:
            tb[sorted_x[i][0]] = labels[i]
            i += 1
        l = data['group'].map(tb)
        return l


import pandas
import datetime
import MySQLdb


class Database:
    def __init__(self):
        self.connection = MySQLdb.connect(host='localhost', user=
        'root', passwd='quangbach1', db='moodle')

    def readDataset(self):
        sql = 'SELECT * FROM mdl_uet_dataset'
        data = pandas.read_sql(sql=sql, con=self.connection)
        return data

    def checkModelVersion(self, modelname):
        cur = self.connection.cursor()
        sql = 'SELECT * FROM mdl_uet_models WHERE model_name = %s AND status=1 '
        cur.execute(sql, (modelname,))
        version = cur.fetchone()
        cur.close()
        if version is None:
            return 0
        else:
            return version[3]

    def saveModelVersion(self, modelname, type, version):
        cur = self.connection.cursor()
        sql = 'SELECT * FROM mdl_uet_models WHERE model_name = %s AND status=1 '
        cur.execute(sql, (modelname,))
        for row in cur:
            if row is not None:
                sql = 'UPDATE mdl_uet_models SET status=0,modified_at=%s WHERE id=%s'
                cur.execute(sql, (datetime.datetime.now(), row[0],))
        sql = 'INSERT INTO mdl_uet_models (model_name,type,version,status,created_at) VALUES (%s,%s,%s,%s,%s)'
        values = (modelname, type, version, 1, datetime.datetime.now())
        success = cur.execute(sql, values)
        self.connection.commit()
        cur.close()
        return success



from .MachineLearning import UETMachineLearning
from sklearn import preprocessing
from sklearn.linear_model import LinearRegression
import numpy as np


class UETRegressor(UETMachineLearning):

    def __init__(self, modelname='', X=[], y=[],filepath=''):
        super().__init__()
        self.modelname = modelname
        self.X = X
        self.y = y
        self.type = 'regressor'
        self.filepath = filepath

    def fit(self):
        self.model = LinearRegression()
        if len(self.X.shape) == 1:
            self.X = np.reshape(self.X,(-1,1))
        # self.transformData()
        self.model.fit(self.X, self.y)
        return self.model

    def predict(self, X):
        if self.model is None:
            self.loadModel()
        if not X.shape[1]:
            X = np.reshape(X, (-1, 1))
        result = self.model.predict(X)
        return result






import pickle
from abc import ABC
from abc import abstractmethod
from .Database import Database
from sklearn import preprocessing
import os


class UETMachineLearning(ABC):

    def __init__(self):
        self.model = None
        self.modelname = ""
        self.type = ""
        self.X = []
        self.y = []
        self.filepath = ''
        self.scaler = preprocessing.MinMaxScaler()

    def filterData(self):
        pass

    def transformData(self):
        self.X = self.scaler.fit_transform(self.X)
        # self.y = scaler.fit_transform(self.y)
        return self.X

    @abstractmethod
    def fit(self):
        pass

    @abstractmethod
    def predict(self, X):
        pass

    def saveModel(self):
        if self.model is None:
            self.fit()
        db = Database()
        version = int(db.checkModelVersion(self.modelname)) + 1
        filename = self.filepath +'/model/' + self.modelname + '_' + self.type + '_' + str(version)
        pickle.dump(self.model, open(filename, 'wb'),protocol=2)
        db.saveModelVersion(self.modelname, self.type, version)
        return self.model

    def loadModel(self):
        db = Database()
        version = db.checkModelVersion(self.modelname)
        if version == 0:
            self.saveModel()
            version = 1
        filename = self.filepath +'/model/' + self.modelname + '_' + self.type + '_' + str(version)
        self.model = pickle.load(open(filename, 'rb'))
        return self.model



from sklearn.linear_model import LinearRegression
import pandas
import operator
import pickle
from matplotlib import pyplot as plt
import numpy as np
from sklearn.cluster import KMeans
from sklearn.mixture import GaussianMixture
from sklearn.mixture import BayesianGaussianMixture
from sklearn.cluster import AgglomerativeClustering
from sklearn.cluster import MeanShift,estimate_bandwidth,SpectralClustering
from UETClustering import UETClustering
from sklearn import metrics
from sklearn.cluster import AgglomerativeClustering
from sklearn.cluster import KMeans
from sklearn.cluster import Birch
from Database import Database
from MoodleBackend import mysql
from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import cross_val_score
from sklearn.preprocessing import MinMaxScaler
# data = pandas.read_csv('830/export3w(1).csv')
# train = data[list(data.columns[4:9])]
# d2 = pandas.read_csv('830/export7w(1).csv')
# plt.scatter(data['view'], d2['view'],  color='black')
# # plt.plot(diabetes_X_test, diabetes_y_pred, color='blue', linewidth=3)
#
#
# plt.show()
# y = data['letter']
# scaler = MinMaxScaler()
# train = scaler.fit_transform(train)
# classification = DecisionTreeClassifier()
# classification.fit(train,y)
# test = pandas.read_csv('830/test.csv');
# print(list(test['letter']))
# test = test[list(test.columns[4:9])]
# test = scaler.fit_transform(test)
# print(classification.predict(test))
# scores = cross_val_score(classification, train, y)
# print(scores.mean())
# from sklearn.neighbors import KNeighborsClassifier
#
# knn = KNeighborsClassifier(n_neighbors=10)
# knn.fit(train,y)
# print('knn')
# print(knn.predict(test))
#
# scores = cross_val_score(knn, train, y)
# print(scores.mean())
# from sklearn.naive_bayes import GaussianNB,MultinomialNB
#
# gauss = GaussianNB()
# gauss.fit(train,y)
# print('naive')
# print(gauss.predict(test))
#
# scores = cross_val_score(gauss, train, y)
# print(scores.mean())
# multi = MultinomialNB()
# multi.fit(train,y)
# print('multi')
# print(multi.predict(test))
#
# scores = cross_val_score(multi, train, y)
# print(scores.mean())
# from sklearn.svm import SVC,NuSVC,LinearSVC
# svc  = SVC()
# svc.fit(train,y)
# li = LinearSVC()
# li.fit(train,y)
# print('svc')
# print(svc.predict(test))
#
# scores = cross_val_score(svc, train, y)
# print(scores.mean())
# print(li.predict(test))
#
# scores = cross_val_score(li, train, y)
# print(scores.mean())
# from sklearn.linear_model import SGDClassifier
# # clf = SGDClassifier(loss="hinge")
# # clf.fit(train, y)
# # print('sgd')
# # print(clf.predict(test))
#
# from sklearn.ensemble import RandomForestClassifier
# random = RandomForestClassifier(n_estimators=100)
# random.fit(train,y)
# print('random')
# print(random.predict(test))
#
# scores = cross_val_score(random, train, y)
# print(scores.mean())
# from sklearn.ensemble import ExtraTreesClassifier,AdaBoostClassifier
# extra = ExtraTreesClassifier(n_estimators=10, max_depth=None,min_samples_split=2, random_state=0)
# extra.fit(train,y)
# print('extra')
# print(extra.predict(test))
#
#
#
# scores = cross_val_score(extra, train, y)
# print(scores.mean())

# kmean = KMeans(n_clusters=5)
# kmean.fit(train)
# print(train)
# df = pandas.DataFrame(train)
# data['group'] = kmean.labels_
# data.to_csv('/home/bachnguyen/Desktop/test.csv')

# hie = AgglomerativeClustering(n_clusters=5)
# hie.fit(train)
# kmean = KMeans(n_clusters=5,algorithm='elkan')
# kmean.fit(train)
# birch = Birch(n_clusters=5)
# birch.fit(train)
#
#
# gauss = GaussianMixture(n_components=5)
# gauss.fit(train)
# # data['group'] = gauss.predict(train)
# # data.to_csv('/home/bachnguyen/Desktop/resultexportgauss3w.csv',',')
#
# bg = BayesianGaussianMixture(n_components=5)
# bg.fit(train)
#
# print('Silhouette Coefficient')
# print(metrics.silhouette_score(train, kmean.labels_, metric='euclidean'))
# print(metrics.silhouette_score(train, hie.labels_, metric='euclidean'))
# print(metrics.silhouette_score(train, birch.labels_, metric='euclidean'))
# print(metrics.silhouette_score(train, gauss.predict(train), metric='euclidean'))
# print(metrics.silhouette_score(train, bg.predict(train), metric='euclidean'))
#
#
#
#
# print('Calinski-Harabaz Index')
# print(metrics.calinski_harabaz_score(train, kmean.labels_) )
# print(metrics.calinski_harabaz_score(train, hie.labels_) )
# print(metrics.calinski_harabaz_score(train, birch.labels_) )
# print(metrics.calinski_harabaz_score(train, gauss.predict(train)) )
# print(metrics.calinski_harabaz_score(train, bg.predict(train)) )



# db = Database(mysql)
# data = db.readDataset()
# dataset = pandas.DataFrame(data[['15w_view', '15w_post', '15w_forumview', '15w_forumpost', '15w_successsubmission','15w_grade']])

#
# hie = AgglomerativeClustering(n_clusters=5)
# hie.fit(dataset)
# kmean = KMeans(n_clusters=5)
# kmean.fit(dataset)
# birch = Birch(n_clusters=5)
# birch.fit(dataset)
# print('Silhouette Coefficient')
# print(metrics.silhouette_score(dataset, kmean.labels_, metric='euclidean'))
# print(metrics.silhouette_score(dataset, hie.labels_, metric='euclidean'))
# print(metrics.silhouette_score(dataset, birch.labels_, metric='euclidean'))
#
# print('Calinski-Harabaz Index')
# print(metrics.calinski_harabaz_score(dataset, kmean.labels_) )
# print(metrics.calinski_harabaz_score(dataset, hie.labels_) )
# print(metrics.calinski_harabaz_score(dataset, birch.labels_) )


# dataset['group'] = kmean.labels_
# dataset.to_csv('/home/bachnguyen/Desktop/a.csv')
#








# bg = BayesianGaussianMixture(n_components=3)
# bg.fit(train)
# data['group'] = bg.predict(train)
# data.to_csv('model/cluster/15wgauss.csv',',')
#
# kmean = KMeans(n_clusters=3)
# kmean.fit(train)
#
#
# data['group'] = kmean.labels_
# distances = kmean.transform(train)
# i = 0
# d = []
# for group in kmean.labels_:
#     d.append(distances[i][group])
#     i+=1
#
# data['distance'] = d
# data.to_csv('model/cluster/15w.csv')

# ag = AgglomerativeClustering(n_clusters=3)
# ag.fit(train)
# data['group'] = ag.labels_
# data.to_csv('model/cluster/15whi.csv')

# bandwidth = estimate_bandwidth(train, quantile=0.2, n_samples=500)
# ms = MeanShift(bandwidth=bandwidth,bin_seeding=True)
# ms.fit(train)
# data['group'] = ms.labels_
# data.to_csv('model/cluster/15wmeanshift.csv')

# spectral = SpectralClustering(n_clusters=3)
# spectral.fit(train)
# data['group'] = spectral.labels_
# data.to_csv('model/cluster/85915wspectral.csv')

#
connection = mysql.connect()
cursor = connection.cursor()
w3 = pandas.read_csv('830/export3w(1).csv')
w3 = w3.drop('grade', 1)
w6 = pandas.read_csv('830/export6w(1).csv')
w6 = w6.drop('grade', 1)
w7 = pandas.read_csv('830/export7w(1).csv')
w7 = w7.drop('grade', 1)
w10 = pandas.read_csv('830/export10w(1).csv')
w10 = w10.drop('grade', 1)
w13 = pandas.read_csv('830/export13w(1).csv')
w13 = w13.drop('grade', 1)
w15 = pandas.read_csv('830/export15w(1).csv')
w15 = w15.drop('grade', 1)
dataframe = pandas.DataFrame()
r = {}
test = w3.merge(w6, on=['stt', 'courseid', 'userid', 'studentName'], suffixes=('_3w', '_6w'))
test = test.merge(w7, on=['stt', 'courseid', 'userid', 'studentName'], suffixes=('_7w', '_7w'))
test = test.merge(w10, on=['stt', 'courseid', 'userid', 'studentName'], suffixes=('', '_10w'))
test = test.merge(w13, on=['stt', 'courseid', 'userid', 'studentName'], suffixes=('', '_13w'))
test = test.merge(w15, on=['stt', 'courseid', 'userid', 'studentName'], suffixes=('', '_15w'))
wg = pandas.read_csv('830/export7w.csv')
wg = wg[['stt', 'courseid', 'userid', 'studentName','grade']]
test = test.merge(wg,on=['stt', 'courseid', 'userid', 'studentName'],suffixes=('', '_7w'))
# cursor.execute('select count(*) from uet_dataset')
wg = pandas.read_csv('830/export15w.csv')
wg = wg[['stt', 'courseid', 'userid', 'studentName','grade']]
test = test.merge(wg,on=['stt', 'courseid', 'userid', 'studentName'],suffixes=('', '_15w'))
# rc = cursor.fetchone()
test = test.drop(['stt', 'studentName'],1)
for index, row in test.iterrows():
    num =  index
    sql = 'INSERT INTO mdl_uet_dataset VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)'
    values = [num]
    values += row.tolist()
    print(sql)
    cursor.execute(sql,tuple(values))
connection.commit()



import pandas as pd
import pickle
from .UETRegressor import UETRegressor
from .UETClustering import UETClustering
from .UETClassification import UETClassification
from .Database import Database
import json
import os


class UETAnalytics:

    def __init__(self,filepath):
        self.dataset = []
        self.filepath = filepath

    # load data from db
    def loadData(self, nrow=10000):
        db = Database()
        self.dataset = db.readDataset()

    def loadModel(self, modelname, type):
        db = Database()
        version = db.checkModelVersion(modelname)
        filename = self.filepath + '/model/' + modelname + '_' + type + '_' + str(version)
        return pickle.load(open(filename, 'rb'))

    def createModel(self):
        self.loadData()
        columns = ['view', 'post', 'forumview', 'forumpost', 'successsubmission']
        weeks = ['3', '6', '10', '13']
        for week in weeks:
            for column in columns:
                modelname = week + 'w_15w_' + column
                self.regression(modelname, 'w' + week + '_' + column, 'w15_' + column)
                if int(week) < 7:
                    modelname = week + 'w_7w_' + column
                    self.regression(modelname, 'w' + week + '_' + column, 'w7_' + column)
        week7 = self.clustering('7w',
                                ['w7_view', 'w7_post', 'w7_forumview', 'w7_forumpost', 'w7_successsubmission',
                                 'w7_grade'])
        week15 = self.clustering('15w',
                                 ['w15_view', 'w15_post', 'w15_forumview', 'w15_forumpost', 'w15_successsubmission',
                                  'w15_grade'])
        self.classifcation('15w',
                           ['view', 'post', 'forumview', 'forumpost', 'successsubmission'], 'group',
                           week15)
        self.classifcation('7w',
                           ['view', 'post', 'forumview', 'forumpost', 'successsubmission'], 'group',
                           week7)

    def predict(self, week, data):
        columns = ['view', 'post', 'forumview', 'forumpost', 'successsubmission']
        predictdata = {}
        regressiondata = {}
        if week < 7:
            for column in columns:
                modelname = str(week) + 'w_7w_' + column
                model = self.loadModel(modelname=modelname, type='regressor')
                regressiondata[column] = model.predict(data[column])[0]
            model = self.loadModel('7w', 'classification')
            temp = [[regressiondata['view'], regressiondata['post'], regressiondata['forumview'],
                     regressiondata['forumpost'], regressiondata['successsubmission']]]
            predictdata['w7'] = model.predict(temp)[0]
        if week == 7:
            model = self.loadModel('7w', 'classification')
            temp = [[data['view'], data['post'], data['forumview'],
                     data['forumpost'], data['successsubmission']]]
            predictdata['w7'] = model.predict(temp)[0]
        if week < 15:
            for column in columns:
                modelname = str(week) + 'w_15w_' + column
                model = self.loadModel(modelname=modelname, type='regressor')
                regressiondata[column] = model.predict(data[column])[0]
            model = self.loadModel('15w', 'classification')
            temp = [[regressiondata['view'], regressiondata['post'], regressiondata['forumview'],
                     regressiondata['forumpost'], regressiondata['successsubmission']]]
            predictdata['w15'] = model.predict(temp)[0]
        if week == 15:
            model = self.loadModel('15w', 'classification')
            temp = [[data['view'], data['post'], data['forumview'],
                     data['forumpost'], data['successsubmission']]]
            predictdata['w15'] = model.predict(temp)[0]
        return json.dumps(predictdata)

    def regression(self, modelname, columnX, columnY):
        X = self.dataset[columnX]
        y = self.dataset[columnY]
        uet = UETRegressor(modelname=modelname, X=X, y=y,filepath=self.filepath)
        uet.fit()
        uet.saveModel()

    def classifcation(self, modelname, columnX, columnY, traindata=[]):
        X = traindata[columnX]
        y = traindata[columnY]
        uet = UETClassification(modelname=modelname, X=X, y=y,filepath=self.filepath)
        uet.fit()
        uet.saveModel()

    def clustering(self, modelname, columnXs):
        X = self.dataset[columnXs]
        uet = UETClustering(n_cluster=3, X=X, modelname=modelname,filepath=self.filepath)
        uet.saveModel()
        best = uet.labelGrade(["F","B","A"])
        clusterF = best[best.group == "F"]
        clusterB = best[best.group == "B"]
        clusterA = best[best.group == "A"]
        cluster = UETClustering(n_cluster=2,X = clusterB)
        cluster.X = clusterB[clusterB.columns[0:6]]
        cluster.fit()
        clusterB = cluster.labelGrade(["D","C"])
        clusters = UETClustering(n_cluster=2, X=clusterA)
        clusters.X = clusterA[clusterA.columns[0:6]]
        clusters.fit()
        clusterA = clusters.labelGrade(["B", "A"])
        best = clusterF.append(clusterA).append(clusterB)
        return best



from setuptools import setup
def readme():
    with open('README.rst') as f:
        return f.read()



setup(name='uetla',
      version='0.6',
      description='UET Analytic for Moodle',
      long_description=readme(),
      keywords='Learning analytics',
      url='https://github.com/NguyenBach/UETBackend.git',
      author='Bach Nguyen',
      author_email='bachnq214@gmail.com',
      packages=['uetla'],
      install_requires=[
          'scikit-learn >= 0.19.1',
          'pandas >= 0.22.0',
          'mysqlclient >= 1.3.12',
          'scipy >= 1.0.0'
      ],
      include_package_data=True,
      zip_safe=False)